unit RegUtils;

interface
uses
  Windows, Registry;

type
  TVBReg=class
  protected
   function CreateHandler(const path:string):TObject;
   procedure DoWrite(const path:string; const value:OleVariant);
  public
   destructor Destroy;override;
  public
   constructor Create;
  public
   function Read(const path:string):OleVariant;
   function Write(const path:string; const value:OleVariant):boolean;
   function Delete(const path:string):boolean;
   function Exists(const path:string):boolean;
   property Values[const path:string]:OleVariant read Read write DoWrite;default;
  end;

 { TVBReg shortcuts }
 function RegRead(const path:string):OleVariant;overload;
 function RegRead(const path:string; const defaultValue:OleVariant):OleVariant;overload;
 function RegWrite(const path:string; const value:OleVariant):boolean;
 function RegDelete(const path:string):boolean;
 function RegExists(const path:string):boolean;

 function RegReadString(const path:string; const defaultValue: string = ''):string;
 function RegReadBool(const path:string; const defaultValue: Boolean = False):Boolean;
 function RegReadInteger(const path:string; const defaultValue: Integer = 0):Integer;

 { registry path helpers }
 function SetCurrentKey(reg:TRegistry; const path:string; Mode:Integer):boolean;
 function GetCurrentKey(reg:TRegistry):string;

{ examples using VBReg

 // write to default value
 RegWrite('HKCU\Software\First Information Systems\',3)

 // create keys and write text value
 RegWrite('HKLM\Software\Microsoft\Windows\CurrentVersion\Uninstall\ActiveChart\DisplayName','hallo 3D')

 // remove associations for files with KUK extension. (both ext key and its shell
 // definition key)
 if RegExists('HKCR\.kuk\') then begin
  RegDelete('HKCR\'+RegRead('HKCR\.kuk\')+'\')
  RegDelete('HKCR\.kuk\')
 end

 // delete value
 RegDelete('HKCU\Software\First Information Systems\ActiveChart\OpenFilePath')
 // delete value by assignment
 RegWrite('HKCU\Software\First Information Systems\ActiveChart\OpenFilePath',Unassigned)

 var reg1,reg2:TRegistry;

 // navigate reg1 worker to given position, create key if necessary
 SetCurrentKey(reg1,'HKCU\Software\First Information Systems\',fmOpenReadWrite)

 // navigate reg2 worker at the same place where worker reg1 currently stands and
 // request read-only access
 SetCurrentKey(reg2,GetCurrentKey(reg1),fmOpenRead)

}

implementation
uses
  Classes, SysUtils, Variants;

constructor TVBReg.Create;
begin
 inherited Create
end;

destructor TVBReg.Destroy;
begin
 inherited
end;

type
 { this is a VB helper class. after instantiation just one of declared functions
   may be run. }
 TReg=class
 private
  r:TRegistry;
  path:string;
  name:string;
  key:HKEY;
  keyName:string;
 public
  constructor Create(APath:string);
  destructor Destroy;override;

  function Exists:boolean;
  function Remove:boolean;
  function GetValue:OleVariant;
  function SetValue(const value:OleVariant):boolean;
 end;

 procedure Parse(const APath:string; var path:string; var key:HKEY; var keyName:string; var name:string);
 var p:integer;
 begin
  path:=APath;
  p:=Pos('\',path);
  key:=0;
  keyName:='';
  if p<>0 then begin
   name:=UpperCase(Copy(path,1,p-1));
   delete(path,1,p);

   keyName:=name+'\';
   if name='HKCR' then
    key:=HKEY_CLASSES_ROOT
   else if name='HKCU' then
    key:=HKEY_CURRENT_USER
   else if name='HKLM' then
    key:=HKEY_LOCAL_MACHINE
   else if name='HKU' then
    key:=HKEY_USERS
  end;

  name:='';
  p:=Pos('\\',path);
  if p=0 then begin
   for p:=length(path) downto 1 do
    if path[p]='\' then begin
     name:=Copy(path,p+1,maxint);
     SetLength(path,p-1);
     break
    end
  end else begin
   name:=Copy(path,p+2,maxint);
   SetLength(path,p-1)
  end
 end;

 constructor TReg.Create;
 begin
  r:=TRegistry.Create;
  Parse(APath,path,key,keyName,name)
 end;

 destructor TReg.Destroy;
 begin
  r.Free;
  inherited
 end;

 function TReg.Exists;
 begin
  Result:=false;
  if key=0 then exit;
  r.RootKey:=key;

  if not r.OpenKeyReadOnly(path) then exit;
  Result:=(name='') or r.ValueExists(name)
 end;

 function TReg.GetValue;
 begin
  if key=0 then exit;
  r.RootKey:=key;

  if path<>'' then if not r.OpenKeyReadOnly(path) then exit;
  if not r.ValueExists(name) then exit;

  case r.GetDataType(name) of
   rdString:  Result:=r.ReadString(name);
   rdInteger: Result:=r.ReadInteger(name);
   // TODO: read back other value types
  end;
 end;

 function TReg.Remove;
 begin
  Result:=true;
  if key=0 then exit;
  r.RootKey:=key;

  if name<>'' then begin
   if not r.OpenKey(path,false) then exit;
   r.DeleteValue(name)
  end else begin
   if r.KeyExists(path) then
    Result:=r.DeleteKey(path)
  end
 end;

 function TReg.SetValue;
 begin
  Result:=false;
  if key=0 then exit;
  r.RootKey:=key;

  if path<>'' then if not r.OpenKey(path,true) then exit;

  case VarType(value) of
   varEmpty,
   varNull:
    begin
     r.DeleteValue(name);
     Result:=true
    end;

   varSmallint,
   varInteger,
   varByte:
    begin
     r.WriteInteger(name,value);
     Result:=true
    end;

   varOleStr,
   varString:
    begin
     r.WriteString(name,value);
     Result:=true
    end;

   varSingle,
   varDouble,
   varCurrency:
    begin
     r.WriteFloat(name,value);
     Result:=true
    end;

   varBoolean:
    begin
     r.WriteBool(name,value);
     Result:=true
    end;

   varDate:
    begin
     r.WriteDate(name,value);
     Result:=true
    end;

{   varDispatch = $0009;
   varError    = $000A;
   varVariant  = $000C;
   varUnknown  = $000D;
   varTypeMask = $0FFF;
   varArray    = $2000;
   varByRef    = $4000; }
  end
 end;

function TVBReg.CreateHandler;
begin
 Result:=TReg.Create(path)
end;

function TVBReg.Read(const path:string):OleVariant;
var r:TReg;
begin
 try
  r:=TReg(CreateHandler(path));
  try
   Result:=r.GetValue
  finally
   r.Free
  end
 except
  Result:=Unassigned
 end
end;

function TVBReg.Write(const path:string; const value:OleVariant):boolean;
var r:TReg;
begin
 try
  r:=TReg(CreateHandler(path));
  try
   Result:=r.SetValue(value)
  finally
   r.Free
  end
 except
  Result:=false
 end
end;

procedure TVBReg.DoWrite;
begin
 if not Write(path,value) then { TODO: raise exception ? }
end;

function TVBReg.Delete(const path:string):boolean;
var r:TReg;
begin
 try
  r:=TReg(CreateHandler(path));
  try
   Result:=r.Remove
  finally
   r.Free
  end
 except
  Result:=false
 end
end;

function TVBReg.Exists(const path:string):boolean;
var r:TReg;
begin
 try
  r:=TReg(CreateHandler(path));
  try
   Result:=r.Exists
  finally
   r.Free
  end
 except
  Result:=false
 end
end;

function RegRead(const path:string; const defaultValue:OleVariant):OleVariant;
var r:TVBReg;
begin
 r:=TVBReg.Create;
 Result:=r.Read(path);
 if VarType(Result) in [varNull, varEmpty] then
  Result:=defaultValue; 
 r.Free
end;

function RegRead(const path:string):OleVariant;
var r:TVBReg;
begin
 r:=TVBReg.Create;
 Result:=r.Read(path);
 r.Free
end;

function RegReadString(const path:string; const defaultValue: string):string;
begin
 try
  Result:=string(RegRead(path,defaultValue))
 except
  Result:=defaultValue
 end
end;

function RegReadInteger(const path:string; const defaultValue: Integer):Integer;
begin
 try
  Result:=Integer(RegRead(path,defaultValue))
 except
  Result:=defaultValue
 end
end;

function RegReadBool(const path:string; const defaultValue: Boolean):Boolean;
begin
 try
  Result:=Boolean(RegRead(path,defaultValue))
 except
  Result:=defaultValue
 end
end;

function RegWrite(const path:string; const value:OleVariant):boolean;
var r:TVBReg;
begin
 r:=TVBReg.Create;
 Result:=r.Write(path,value);
 r.Free
end;

function RegDelete(const path:string):boolean;
var r:TVBReg;
begin
 r:=TVBReg.Create;
 Result:=r.Delete(path);
 r.Free
end;

function RegExists(const path:string):boolean;
var r:TVBReg;
begin
 r:=TVBReg.Create;
 Result:=r.Exists(path);
 r.Free
end;

function SetCurrentKey;
var keyPath,keyName,name:string;
var key:HKEY;
var flags:set of (fCanCreate,fReadOnly);
begin
 Parse(path,keyPath,key,keyName,name);
 Result:=false;
 if (length(name)>0) or (key=0) then exit;
 try
  reg.RootKey:=key;
  if Mode=fmCreate then
   flags:=[fCanCreate]
  else case Mode and $F of
   fmOpenRead: flags:=[fReadOnly];
   fmOpenReadWrite: flags:=[fCanCreate];
   fmOpenWrite: flags:=[]
   else assert(false)
  end;
  // Result:=reg.OpenKeyEx(keyPath,fCanCreate in flags,fReadOnly in flags)
  if fReadOnly in flags then
   Result:=reg.OpenKeyReadOnly(keyPath)
  else
   Result:=reg.OpenKey(keyPath,fCanCreate in flags)
 except
 end
end;

function GetCurrentKey;
begin
 case reg.RootKey of
 HKEY_CURRENT_USER: Result:='HKCU';
 HKEY_USERS: Result:='HKU';
 HKEY_LOCAL_MACHINE: Result:='HKLM';
 HKEY_CLASSES_ROOT: Result:='HKCR'
 else
  Result:=''
 end;
 if length(Result)>0 then Result:=Result+'\'+reg.CurrentPath+'\'
end;

end.
